package com.loafle.overflow.module.member.service;

import com.loafle.overflow.module.member.dao.MemberTotpDAO;
import com.loafle.overflow.module.member.model.Member;
import com.loafle.overflow.module.member.model.MemberTotp;
import com.warrenstrange.googleauth.GoogleAuthenticator;
import com.warrenstrange.googleauth.GoogleAuthenticatorKey;
import com.warrenstrange.googleauth.GoogleAuthenticatorQRGenerator;
import org.apache.http.client.utils.URIBuilder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.Map;

/**
 * Created by geek on 18. 3. 8.
 */
@Service("MemberTotpService")
public class MemberTotpService {
    @Autowired
    private MemberTotpDAO totpDAO;

    public void  regist(Member member, String secretCode, String code) throws Exception {

        if (null == member || 0 < member.getId()) {
            throw new Exception("Not Null Member ID");
        }

        if (!this.checkCode(secretCode, code) ) {
            throw new Exception("Not Equal");
        }

        MemberTotp totp = new MemberTotp();
        totp.setMember(member);
        totp.setSecretCode(secretCode);

        this.totpDAO.save(totp);
    }

    public MemberTotp modify(MemberTotp totp) throws Exception {
        if ( null == totp.getSecretCode() || totp.getSecretCode().equals("")) {
            throw new Exception("Not Null SecretCode");
        }
        if (null == totp.getMember() || 0 < totp.getMember().getId()) {
            throw new Exception("Not Null Member ID");
        }

        return this.totpDAO.save(totp);
    }

    public void remove(long id) throws Exception {
        this.totpDAO.delete(id);
    }

    public MemberTotp read(long id) throws Exception {
        return this.totpDAO.findOne(id);
    }

    public boolean checkCode(String secretCode, String code) throws Exception {
        GoogleAuthenticator googleAuthenticator = new GoogleAuthenticator();
        boolean isCheck = googleAuthenticator.authorize(secretCode, Integer.valueOf(code));

        if (!isCheck) {
            throw new Exception("Invalid Code");
        }

        return isCheck;
    }

    public Map<String, String> createTotp(Member member) {
//        MemberTotp totp = new MemberTotp();
        Map<String, String> returnMap = new HashMap<>();

        GoogleAuthenticator googleAuthenticator = new GoogleAuthenticator();
        final GoogleAuthenticatorKey key = googleAuthenticator.createCredentials();

        String secret = key.getKey();
//        List<Integer> scratchCodes = key.getScratchCodes();
//        String otpAuthURL = GoogleAuthenticatorQRGenerator.getOtpAuthURL("overFlow", member.getEmail(), key);
        URIBuilder uri = (new URIBuilder()).setScheme("otpauth")
                .setHost("totp")
                .setPath("/" + formatLabel("overFlow", member.getEmail()))
                .setParameter("secret", key.getKey());

//        totp.setMember(member);
//        totp.setSecretCode(secret);
//        totp.setOtpAuth(uri.toString());

        returnMap.put("key", secret);
        returnMap.put("uri", uri.toString());

        return returnMap;
    }

    private String formatLabel(String issuer, String accountName) {
        if(accountName != null && accountName.trim().length() != 0) {
            StringBuilder sb = new StringBuilder();
            if(issuer != null) {
                if(issuer.contains(":")) {
                    throw new IllegalArgumentException("Issuer cannot contain the ':' character.");
                }

                sb.append(issuer);
                sb.append(":");
            }

            sb.append(accountName);
            return sb.toString();
        } else {
            throw new IllegalArgumentException("Account name must not be empty.");
        }
    }
}
